@@ -41,6 +41,7 @@
 #include <kmessagebox.h>
 //Added by qt3to4:
 #include <QShowEvent>
+#include <QLabel>
 
 #include <QCheckBox>
 #include <QVBoxLayout>
@@ -51,20 +52,26 @@
 #include <kauthorized.h>
 
 K_PLUGIN_FACTORY(KateKonsoleFactory, registerPlugin<KateKonsolePlugin>();)
-K_EXPORT_PLUGIN(KateKonsoleFactory(KAboutData("katekonsoleplugins","katekonsoleplugin",ki18n("Konsole"), "0.1", ki18n("Embedded Konsole"), KAboutData::License_LGPL_V2)) )
+K_EXPORT_PLUGIN(KateKonsoleFactory(KAboutData("katekonsole","katekonsoleplugin",ki18n("Konsole"), "0.1", ki18n("Embedded Konsole"), KAboutData::License_LGPL_V2)) )
 
 KateKonsolePlugin::KateKonsolePlugin( QObject* parent, const QList<QVariant>& ):
     Kate::Plugin ( (Kate::Application*)parent )
 {
+  m_previousEditorEnv=qgetenv("EDITOR");
   if (!KAuthorized::authorizeKAction("shell_access"))
   {
     KMessageBox::sorry(0, i18n ("You do not have enough karma to access a shell or terminal emulation"));
   }
 }
 
+KateKonsolePlugin::~KateKonsolePlugin()
+{
+  ::setenv( "EDITOR", m_previousEditorEnv.data(), 1 );
+}
+
 Kate::PluginView *KateKonsolePlugin::createView (Kate::MainWindow *mainWindow)
 {
-  KateKonsolePluginView *view = new KateKonsolePluginView (mainWindow);
+  KateKonsolePluginView *view = new KateKonsolePluginView (this,mainWindow);
   mViews.append( view );
   return view;
 }
@@ -101,12 +108,12 @@ void KateKonsolePlugin::readConfig()
     view->readConfig();
 }
 
-KateKonsolePluginView::KateKonsolePluginView (Kate::MainWindow *mainWindow)
-    : Kate::PluginView (mainWindow)
+KateKonsolePluginView::KateKonsolePluginView (KateKonsolePlugin* plugin, Kate::MainWindow *mainWindow)
+    : Kate::PluginView (mainWindow),m_plugin(plugin)
 {
   // init console
   QWidget *toolview = mainWindow->createToolView ("kate_private_plugin_katekonsoleplugin", Kate::MainWindow::Bottom, SmallIcon("utilities-terminal"), i18n("Terminal"));
-  m_console = new KateConsole(mainWindow, toolview);
+  m_console = new KateConsole(m_plugin, mainWindow, toolview);
 }
 
 KateKonsolePluginView::~KateKonsolePluginView ()
@@ -122,30 +129,28 @@ void KateKonsolePluginView::readConfig()
   m_console->readConfig();
 }
 
-KateConsole::KateConsole (Kate::MainWindow *mw, QWidget *parent)
-    : KVBox (parent), KXMLGUIClient()
+KateConsole::KateConsole (KateKonsolePlugin* plugin, Kate::MainWindow *mw, QWidget *parent)
+    : KVBox (parent), Kate::XMLGUIClient(KateKonsoleFactory::componentData())
     , m_part (0)
     , m_mw (mw)
     , m_toolView (parent)
+    , m_plugin(plugin)
 {
-  // this must be called before putting anything into actionCollection()
-  setComponentData (KComponentData("kate"));
 
   QAction* a = actionCollection()->addAction("katekonsole_tools_pipe_to_terminal");
-  a->setIcon(KIcon("pipe"));
-  a->setText(i18n("&Pipe to Terminal"));
+  a->setIcon(KIcon("utilities-terminal"));
+  a->setText(i18nc("@action", "&Pipe to Terminal"));
   connect(a, SIGNAL(triggered()), this, SLOT(slotPipeToConsole()));
 
   a = actionCollection()->addAction("katekonsole_tools_sync");
-  a->setText(i18n("S&ynchronize Terminal with Current Document"));
+  a->setText(i18nc("@action", "S&ynchronize Terminal with Current Document"));
   connect(a, SIGNAL(triggered()), this, SLOT(slotManualSync()));
 
   a = actionCollection()->addAction("katekonsole_tools_toggle_focus");
   a->setIcon(KIcon("utilities-terminal"));
-  a->setText(i18n("&Focus Terminal"));
+  a->setText(i18nc("@action", "&Focus Terminal"));
   connect(a, SIGNAL(triggered()), this, SLOT(slotToggleFocus()));
 
-  setXMLFile("plugins/katekonsole/ui.rc");
   m_mw->guiFactory()->addClient (this);
 
   readConfig();
@@ -288,10 +293,15 @@ void KateConsole::slotToggleFocus()
 
 void KateConsole::readConfig()
 {
+  disconnect( m_mw, SIGNAL(viewChanged()), this, SLOT(slotSync()) );
   if ( KConfigGroup(KGlobal::config(), "Konsole").readEntry("AutoSyncronize", false) )
     connect( m_mw, SIGNAL(viewChanged()), SLOT(slotSync()) );
+    
+  
+  if ( KConfigGroup(KGlobal::config(), "Konsole").readEntry("SetEditor", false) )
+    ::setenv( "EDITOR", "kate -b",1);
   else
-    disconnect( m_mw, SIGNAL(viewChanged()), this, SLOT(slotSync()) );
+    ::setenv( "EDITOR", m_plugin->previousEditorEnv().data(), 1 );
 }
 
 KateKonsoleConfigPage::KateKonsoleConfigPage( QWidget* parent, KateKonsolePlugin *plugin )
@@ -303,15 +313,22 @@ KateKonsoleConfigPage::KateKonsoleConfigPage( QWidget* parent, KateKonsolePlugin
 
   cbAutoSyncronize = new QCheckBox( i18n("&Automatically synchronize the terminal with the current document when possible"), this );
   lo->addWidget( cbAutoSyncronize );
+  cbSetEditor = new QCheckBox( i18n("Set &EDITOR environment variable to 'kate -b'"), this );
+  lo->addWidget( cbSetEditor );
+  QLabel *tmp = new QLabel(this);
+  tmp->setText(i18n("Important: The document has to be closed to make the console application continue"));
+  lo->addWidget(tmp);
   reset();
   lo->addStretch();
   connect( cbAutoSyncronize, SIGNAL(stateChanged(int)), SIGNAL(changed()) );
+  connect( cbSetEditor, SIGNAL(stateChanged(int)), SIGNAL(changed()) );
 }
 
 void KateKonsoleConfigPage::apply()
 {
   KConfigGroup config(KGlobal::config(), "Konsole");
   config.writeEntry("AutoSyncronize", cbAutoSyncronize->isChecked());
+  config.writeEntry("SetEditor", cbSetEditor->isChecked());
   config.sync();
   mPlugin->readConfig();
 }
@@ -320,6 +337,7 @@ void KateKonsoleConfigPage::reset()
 {
   KConfigGroup config(KGlobal::config(), "Konsole");
   cbAutoSyncronize->setChecked(config.readEntry("AutoSyncronize", false));
+  cbSetEditor->setChecked(config.readEntry("SetEditor", false));
 }
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
