@@ -47,12 +47,11 @@ public:
     /**
      *  Sets up a ClassPropDlg.
      *
-     *  @param  parent  The parent of the ClassPropDlg
-     *  @param  c       The UMLObject to display properties of.
-     *  @param  pageNum The page to show first.
-     *  @param assoc    Determines whether to display associations
+     *  @param parent    The parent of the ClassPropDlg
+     *  @param c         The UMLObject to display properties of.
+     *  @param assoc     Determines whether to display associations
      */
-    ClassPropDlg(QWidget *parent, UMLObject *c, int pageNum = 0, bool assoc = false);
+    ClassPropDlg(QWidget *parent, UMLObject *c, bool assoc = false);
 
     /**
      *  Sets up a ClassPropDlg.
@@ -81,86 +80,38 @@ public:
 
 protected slots:
 
-    /**
-     * Calls slotApply() and accepts (closes) the dialog.
-     */
     void slotOk();
 
-    /**
-     * Applys the settings in the dialog to the widget and object.
-     */
     void slotApply();
 
 protected:
 
-    /**
-     * Sets up the general, attribute, operations, template and association pages as appropriate.
-     */
     void setupPages(bool assoc = false);
 
-    /**
-     * Sets up the page "General" for the component.
-     */
     void setupGeneralPage();
 
-    /**
-     * Sets up the page "Color" for the component.
-     */
     void setupColorPage();
 
-    /**
-     * Sets up the page "Display" for the component.
-     */
     void setupDisplayPage();
 
-    /**
-     * Sets up the page "Attributes" for the component.
-     */
     void setupAttributesPage();
 
-    /**
-     * Sets up the page "Operations" for the component.
-     */
     void setupOperationsPage();
 
-    /**
-     * Sets up the page "Templates" for the component.
-     */
     void setupTemplatesPage();
 
-    /**
-     * Sets up the page "Enum Literals" for the component.
-     */
     void setupEnumLiteralsPage();
 
-    /**
-     * Sets up the page "Entity Attributes" for the component.
-     */
     void setupEntityAttributesPage();
 
-    /**
-     * Sets up the page "Entity Constraints" for the component.
-     */
     void setupEntityConstraintsPage();
 
-    /**
-     * Sets up the page "Contents" for the component.
-     */
     void setupContentsPage();
 
-    /**
-     * Sets up the page "Associations" for the component.
-     */
     void setupAssociationsPage();
 
-    /**
-     * Sets up the general page for the component.
-     */
     void setupInstancePages();
 
-    /**
-     * Sets up the font page.
-     */
     void setupFontPage();
 
 private:
